@@ -26,19 +26,31 @@
 /*
   somewhat ugly (those *** :) so I feel like I may comment
   as first argument it takes poiter to pointer on Dialog struct
-  which MUST contains GnomeDialog * as first field
+  which MUST contains GtkDialog * as first field
 
   it is bassicaly the same trick as used in GtkObject's
 */
 
+GtkWindow *
+get_parent_window (GtkWidget *w)
+{
+	while (w && ! GTK_IS_WINDOW (w))
+		w = w->parent;
+
+	return w ? GTK_WINDOW (w) : NULL;
+}
+
 void
-run_dialog (GnomeDialog ***dialog, GtkHTML *html, DialogCtor ctor)
+run_dialog (GtkDialog ***dialog, GtkHTML *html, GtkHTMLControlData *cd, DialogCtor ctor, const gchar *title)
 {
 	if (*dialog) {
+		gtk_window_set_title (GTK_WINDOW (**dialog), title);
 		gtk_widget_show (GTK_WIDGET (**dialog));
 		gdk_window_raise (GTK_WIDGET (**dialog)->window);
 	} else {
-		*dialog = ctor (html);
+		*dialog = ctor (html, cd);
+		gtk_window_set_title (GTK_WINDOW (**dialog), title);
 		gtk_widget_show (GTK_WIDGET (**dialog));
 	}
+	gtk_dialog_run (**dialog);
 }
