@@ -21,6 +21,7 @@
 */
 
 #include <config.h>
+#include <string.h>
 #include <bonobo.h>
 
 #include "gtkhtml.h"
@@ -32,9 +33,9 @@
 #include "htmlundo.h"
 
 #include "engine.h"
+#include "spell.h"
 
 static BonoboObjectClass *engine_parent_class;
-static POA_GNOME_GtkHTML_Editor_Engine__vepv engine_vepv;
 
 inline static EditorEngine *
 html_editor_engine_from_servant (PortableServer_Servant servant)
@@ -48,9 +49,9 @@ impl_get_paragraph_data (PortableServer_Servant servant, const CORBA_char * key,
 	EditorEngine *e = html_editor_engine_from_servant (servant);
 	CORBA_char * value = CORBA_OBJECT_NIL;
 
-	if (e->html->engine->cursor->object && e->html->engine->cursor->object->parent
-	    && HTML_IS_CLUEFLOW (e->html->engine->cursor->object->parent))
-		value = html_object_get_data (e->html->engine->cursor->object->parent, key);
+	if (e->cd->html->engine->cursor->object && e->cd->html->engine->cursor->object->parent
+	    && HTML_IS_CLUEFLOW (e->cd->html->engine->cursor->object->parent))
+		value = html_object_get_data (e->cd->html->engine->cursor->object->parent, key);
 
 	/* printf ("get paragraph data\n"); */
 
@@ -64,9 +65,9 @@ impl_set_paragraph_data (PortableServer_Servant servant,
 {
 	EditorEngine *e = html_editor_engine_from_servant (servant);
 
-	if (e->html->engine->cursor->object && e->html->engine->cursor->object->parent
-	    && HTML_OBJECT_TYPE (e->html->engine->cursor->object->parent) == HTML_TYPE_CLUEFLOW)
-		html_object_set_data (HTML_OBJECT (e->html->engine->cursor->object->parent), key, value);
+	if (e->cd->html->engine->cursor->object && e->cd->html->engine->cursor->object->parent
+	    && HTML_OBJECT_TYPE (e->cd->html->engine->cursor->object->parent) == HTML_TYPE_CLUEFLOW)
+		html_object_set_data (HTML_OBJECT (e->cd->html->engine->cursor->object->parent), key, value);
 }
 
 static void
@@ -79,14 +80,7 @@ impl_set_object_data_by_type (PortableServer_Servant servant,
 	/* printf ("set data by type\n"); */
 
 	/* FIXME should use bonobo_arg_to_gtk, but this seems to be broken */
-	html_engine_set_data_by_type (e->html->engine, html_type_from_name (type_name), key, value);
-}
-
-static void
-unref_listener (EditorEngine *e)
-{
-	if (e->listener_client != CORBA_OBJECT_NIL)
-		bonobo_object_unref (BONOBO_OBJECT (e->listener_client));
+	html_engine_set_data_by_type (e->cd->html->engine, html_type_from_name (type_name), key, value);
 }
 
 static void
@@ -96,10 +90,8 @@ impl_set_listener (PortableServer_Servant servant, const GNOME_GtkHTML_Editor_Li
 
 	/* printf ("set listener\n"); */
 
-	unref_listener (e);
-	e->listener_client = bonobo_object_client_from_corba (CORBA_Object_duplicate (value, ev));
-	e->listener        = bonobo_object_client_query_interface (e->listener_client,
-								   "IDL:GNOME/GtkHTML/Editor/Listener:1.0", ev);
+	bonobo_object_release_unref (e->listener, NULL);
+	e->listener        = bonobo_object_dup_ref (value, NULL);
 }
 
 static GNOME_GtkHTML_Editor_Listener
@@ -115,7 +107,7 @@ impl_run_command (PortableServer_Servant servant, const CORBA_char * command, CO
 
 	/* printf ("command: %s\n", command); */
 
-	return gtk_html_command (e->html, command);
+	return gtk_html_command (e->cd->html, command);
 }
 
 static CORBA_boolean
@@ -123,10 +115,10 @@ impl_is_paragraph_empty (PortableServer_Servant servant, CORBA_Environment * ev)
 {
 	EditorEngine *e = html_editor_engine_from_servant (servant);
 
-	if (e->html->engine->cursor->object
-	    && e->html->engine->cursor->object->parent
-	    && HTML_OBJECT_TYPE (e->html->engine->cursor->object->parent) == HTML_TYPE_CLUEFLOW) {
-		return html_clueflow_is_empty (HTML_CLUEFLOW (e->html->engine->cursor->object->parent));
+	if (e->cd->html->engine->cursor->object
+	    && e->cd->html->engine->cursor->object->parent
+	    && HTML_OBJECT_TYPE (e->cd->html->engine->cursor->object->parent) == HTML_TYPE_CLUEFLOW) {
+		return html_clueflow_is_empty (HTML_CLUEFLOW (e->cd->html->engine->cursor->object->parent));
 	}
 	return CORBA_FALSE;
 }
@@ -136,11 +128,11 @@ impl_is_previous_paragraph_empty (PortableServer_Servant servant, CORBA_Environm
 {
 	EditorEngine *e = html_editor_engine_from_servant (servant);
 
-	if (e->html->engine->cursor->object
-	    && e->html->engine->cursor->object->parent
-	    && e->html->engine->cursor->object->parent->prev
-	    && HTML_IS_CLUEFLOW (e->html->engine->cursor->object->parent->prev))
-		return html_clueflow_is_empty (HTML_CLUEFLOW (e->html->engine->cursor->object->parent->prev));
+	if (e->cd->html->engine->cursor->object
+	    && e->cd->html->engine->cursor->object->parent
+	    && e->cd->html->engine->cursor->object->parent->prev
+	    && HTML_IS_CLUEFLOW (e->cd->html->engine->cursor->object->parent->prev))
+		return html_clueflow_is_empty (HTML_CLUEFLOW (e->cd->html->engine->cursor->object->parent->prev));
 
 	return CORBA_FALSE;
 }
@@ -149,7 +141,7 @@ static void
 impl_insert_html (PortableServer_Servant servant, const CORBA_char * html, CORBA_Environment * ev)
 {
 	/* printf ("impl_insert_html\n"); */
-	gtk_html_insert_html (html_editor_engine_from_servant (servant)->html, html);
+	gtk_html_insert_html (html_editor_engine_from_servant (servant)->cd->html, html);
 }
 
 static CORBA_boolean
@@ -161,50 +153,100 @@ impl_search_by_data (PortableServer_Servant servant, const CORBA_long level, con
 	gchar *o_value;
 
 	do {
-		if (e->html->engine->cursor->object != lco) {
-			o = html_object_nth_parent (e->html->engine->cursor->object, level);
+		if (e->cd->html->engine->cursor->object != lco) {
+			o = html_object_nth_parent (e->cd->html->engine->cursor->object, level);
 			if (o) {
 				o_value = html_object_get_data (o, key);
 				if (o_value && !strcmp (o_value, value))
 					return TRUE;
 			}
 		}
-		lco = e->html->engine->cursor->object;
-	} while (html_cursor_forward (e->html->engine->cursor, e->html->engine));
+		lco = e->cd->html->engine->cursor->object;
+	} while (html_cursor_forward (e->cd->html->engine->cursor, e->cd->html->engine));
 	return FALSE;
 }
 
 static void
 impl_freeze (PortableServer_Servant servant, CORBA_Environment * ev)
 {
-	html_engine_freeze (html_editor_engine_from_servant (servant)->html->engine);
+	html_engine_freeze (html_editor_engine_from_servant (servant)->cd->html->engine);
 }
 
 static void
 impl_thaw (PortableServer_Servant servant, CORBA_Environment * ev)
 {
-	html_engine_thaw (html_editor_engine_from_servant (servant)->html->engine);
+	html_engine_thaw (html_editor_engine_from_servant (servant)->cd->html->engine);
 }
 
 static void
 impl_undo_begin (PortableServer_Servant servant, const CORBA_char * undo_name, const CORBA_char * redo_name,
 		 CORBA_Environment * ev)
 {
-	html_undo_level_begin (html_editor_engine_from_servant (servant)->html->engine->undo, undo_name, redo_name);
+	html_undo_level_begin (html_editor_engine_from_servant (servant)->cd->html->engine->undo, undo_name, redo_name);
 }
 
 static void
 impl_undo_end (PortableServer_Servant servant, CORBA_Environment * ev)
 {
-	html_undo_level_end (html_editor_engine_from_servant (servant)->html->engine->undo);
+	html_undo_level_end (html_editor_engine_from_servant (servant)->cd->html->engine->undo);
 }
 
-POA_GNOME_GtkHTML_Editor_Engine__epv *
-editor_engine_get_epv (void)
+static void
+impl_ignore_word (PortableServer_Servant servant, const CORBA_char * word, CORBA_Environment * ev)
 {
-	POA_GNOME_GtkHTML_Editor_Engine__epv *epv;
+	EditorEngine *e = html_editor_engine_from_servant (servant);
 
-	epv = g_new0 (POA_GNOME_GtkHTML_Editor_Engine__epv, 1);
+	/* printf ("ignoreWord: %s\n", word); */
+
+	spell_add_to_session (e->cd->html, word, e->cd);
+}
+
+static CORBA_boolean
+impl_has_undo (PortableServer_Servant servant, CORBA_Environment * ev)
+{
+	EditorEngine *e = html_editor_engine_from_servant (servant);
+
+	/* printf ("hasUndo\n"); */
+
+	return gtk_html_has_undo (e->cd->html);
+}
+
+static void
+impl_drop_undo (PortableServer_Servant servant, CORBA_Environment * ev)
+{
+	EditorEngine *e = html_editor_engine_from_servant (servant);
+
+	/* printf ("dropUndo\n"); */
+
+	gtk_html_drop_undo (e->cd->html);
+}
+
+static void
+engine_object_finalize (GObject *object)
+{
+	EditorEngine *e = EDITOR_ENGINE (object);
+
+	bonobo_object_release_unref (e->listener, NULL);
+
+	G_OBJECT_CLASS (engine_parent_class)->finalize (object);
+}
+
+static void
+editor_engine_init (GObject *object)
+{
+	EditorEngine *e = EDITOR_ENGINE (object);
+
+	e->listener = CORBA_OBJECT_NIL;
+}
+
+static void
+editor_engine_class_init (EditorEngineClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	POA_GNOME_GtkHTML_Editor_Engine__epv *epv = &klass->epv;
+
+	engine_parent_class = g_type_class_peek_parent (klass);
+	object_class->finalize = engine_object_finalize;
 
 	epv->_set_listener            = impl_set_listener;
 	epv->_get_listener            = impl_get_listener;
@@ -218,125 +260,27 @@ editor_engine_get_epv (void)
 	epv->insertHTML               = impl_insert_html;
 	epv->freeze                   = impl_freeze;
 	epv->thaw                     = impl_thaw;
-	epv->undo_begin               = impl_undo_begin;
-	epv->undo_end                 = impl_undo_end;
-
-	return epv;
+	epv->undoBegin                = impl_undo_begin;
+	epv->undoEnd                  = impl_undo_end;
+	epv->ignoreWord               = impl_ignore_word;
+	epv->hasUndo                  = impl_has_undo;
+	epv->dropUndo                 = impl_drop_undo;
 }
 
-static void
-init_engine_corba_class (void)
-{
-	engine_vepv.Bonobo_Unknown_epv    = bonobo_object_get_epv ();
-	engine_vepv.GNOME_GtkHTML_Editor_Engine_epv = editor_engine_get_epv ();
-}
-
-static void
-engine_object_init (GtkObject *object)
-{
-	EditorEngine *e = EDITOR_ENGINE (object);
-
-	e->listener_client = CORBA_OBJECT_NIL;
-}
-
-static void
-engine_object_destroy (GtkObject *object)
-{
-	EditorEngine *e = EDITOR_ENGINE (object);
-
-	unref_listener (e);
-
-	GTK_OBJECT_CLASS (engine_parent_class)->destroy (object);
-}
-
-static void
-engine_class_init (EditorEngineClass *klass)
-{
-	GtkObjectClass *object_class = (GtkObjectClass *) klass;
-
-	engine_parent_class   = gtk_type_class (bonobo_object_get_type ());
-	object_class->destroy = engine_object_destroy;
-
-	init_engine_corba_class ();
-}
-
-GtkType
-editor_engine_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type){
-		GtkTypeInfo info = {
-			"EditorEngine",
-			sizeof (EditorEngine),
-			sizeof (EditorEngineClass),
-			(GtkClassInitFunc) engine_class_init,
-			(GtkObjectInitFunc) engine_object_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (bonobo_object_get_type (), &info);
-	}
-
-	return type;
-}
+BONOBO_TYPE_FUNC_FULL (
+	EditorEngine,                  /* Glib class name */
+	GNOME_GtkHTML_Editor_Engine,   /* CORBA interface name */
+	BONOBO_TYPE_OBJECT,            /* parent type */
+	editor_engine);                /* local prefix ie. 'echo'_class_init */
 
 EditorEngine *
-editor_engine_construct (EditorEngine *engine, GNOME_GtkHTML_Editor_Engine corba_engine)
+editor_engine_new (GtkHTMLControlData *cd)
 {
-	g_return_val_if_fail (engine != NULL, NULL);
-	g_return_val_if_fail (IS_EDITOR_ENGINE (engine), NULL);
-	g_return_val_if_fail (corba_engine != CORBA_OBJECT_NIL, NULL);
+	EditorEngine *ee;
 
-	engine->listener_client = CORBA_OBJECT_NIL;
+	ee = g_object_new (EDITOR_ENGINE_TYPE, NULL);
 
-	if (!bonobo_object_construct (BONOBO_OBJECT (engine), (CORBA_Object) corba_engine))
-		return NULL;
+	ee->cd = cd;
 
-	return engine;
-}
-
-static GNOME_GtkHTML_Editor_Engine
-create_engine (BonoboObject *engine)
-{
-	POA_GNOME_GtkHTML_Editor_Engine *servant;
-	CORBA_Environment ev;
-
-	servant = (POA_GNOME_GtkHTML_Editor_Engine *) g_new0 (BonoboObjectServant, 1);
-	servant->vepv = &engine_vepv;
-
-	CORBA_exception_init (&ev);
-	POA_GNOME_GtkHTML_Editor_Engine__init ((PortableServer_Servant) servant, &ev);
-	ORBIT_OBJECT_KEY(servant->_private)->object = NULL;
-
-	if (ev._major != CORBA_NO_EXCEPTION){
-		g_free (servant);
-		CORBA_exception_free (&ev);
-		return CORBA_OBJECT_NIL;
-	}
-
-	CORBA_exception_free (&ev);
-
-	return (GNOME_GtkHTML_Editor_Engine) bonobo_object_activate_servant (engine, servant);
-}
-
-EditorEngine *
-editor_engine_new (GtkHTML *html)
-{
-	EditorEngine *engine;
-	GNOME_GtkHTML_Editor_Engine corba_engine;
-
-	engine = gtk_type_new (EDITOR_ENGINE_TYPE);
-	engine->html = html;
-
-	corba_engine = create_engine (BONOBO_OBJECT (engine));
-
-	if (corba_engine == CORBA_OBJECT_NIL) {
-		bonobo_object_unref (BONOBO_OBJECT (engine));
-		return NULL;
-	}
-	
-	return editor_engine_construct (engine, corba_engine);
+	return ee;
 }
