@@ -20,8 +20,7 @@
     Boston, MA 02111-1307, USA.
 */
 
-#include <config.h>
-#include <gal/widgets/e-unicode.h>
+#include <config.h> 
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include "search.h"
@@ -29,75 +28,81 @@
 #include "htmlengine-search.h"
 
 struct _GtkHTMLSearchDialog {
-	GnomeDialog *dialog;
+	GtkDialog   *dialog;
 	GtkHTML     *html;
 	GtkWidget   *entry;
 	GtkWidget   *backward;
 	GtkWidget   *case_sensitive;
 
-	gboolean     regular;
+	GtkHTMLControlData *cd;
 };
 
-static void
-search_cb (GtkWidget *but, GtkHTMLSearchDialog *d)
-{
-	char *text;
-
-	text = e_utf8_gtk_entry_get_text (GTK_ENTRY (d->entry));
-	html_engine_search (d->html->engine, text,
-			    GTK_TOGGLE_BUTTON (d->case_sensitive)->active,
-			    GTK_TOGGLE_BUTTON (d->backward)->active == 0, d->regular);
-	g_free (text);
-}
-
 static void
 entry_changed (GtkWidget *entry, GtkHTMLSearchDialog *d)
 {
+	if (d->cd->search_text)
+		g_free (d->cd->search_text);
+	d->cd->search_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (d->entry)));
 }
 
 static void
 entry_activate (GtkWidget *entry, GtkHTMLSearchDialog *d)
 {
-	gnome_dialog_close (d->dialog);
-	search_cb (NULL, d);
+	gtk_dialog_response (d->dialog, 0);
+}
+
+static void
+search_dialog_response (GtkDialog *dialog, gint response_id, GtkHTMLSearchDialog *d)
+{
+	switch (response_id) {
+	case 0: /* Search */
+		gtk_widget_hide (GTK_WIDGET (d->dialog));
+		html_engine_search (d->html->engine, gtk_entry_get_text (GTK_ENTRY (d->entry)),
+				    GTK_TOGGLE_BUTTON (d->case_sensitive)->active,
+				    GTK_TOGGLE_BUTTON (d->backward)->active == 0, d->cd->regular);
+		break;
+	}
 }
 
 GtkHTMLSearchDialog *
-gtk_html_search_dialog_new (GtkHTML *html)
+gtk_html_search_dialog_new (GtkHTML *html, GtkHTMLControlData *cd)
 {
 	GtkHTMLSearchDialog *dialog = g_new (GtkHTMLSearchDialog, 1);
 	GtkWidget *hbox;
 
-	dialog->dialog         = GNOME_DIALOG (gnome_dialog_new (NULL, _("Find"), GNOME_STOCK_BUTTON_CANCEL, NULL));
+	dialog->dialog         = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Find"), NULL, 0,
+									  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+									  GTK_STOCK_FIND, 0,
+									  NULL));
 	dialog->entry          = gtk_entry_new ();
-	dialog->backward       = gtk_check_button_new_with_label (_("backward"));
-	dialog->case_sensitive = gtk_check_button_new_with_label (_("case sensitive"));
+	dialog->backward       = gtk_check_button_new_with_mnemonic (_("_Backward"));
+	dialog->case_sensitive = gtk_check_button_new_with_mnemonic (_("Case _sensitive"));
 	dialog->html           = html;
-	dialog->regular        = FALSE;
+	dialog->cd             = cd;
 
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 6);
 
-	gtk_box_pack_start_defaults (GTK_BOX (hbox), dialog->backward);
-	gtk_box_pack_start_defaults (GTK_BOX (hbox), dialog->case_sensitive);
+	if (cd->search_text)
+		gtk_entry_set_text (GTK_ENTRY (dialog->entry), cd->search_text);
 
-	gtk_box_pack_start_defaults (GTK_BOX (dialog->dialog->vbox), dialog->entry);
-	gtk_box_pack_start_defaults (GTK_BOX (dialog->dialog->vbox), hbox);
+	gtk_box_pack_start (GTK_BOX (hbox), dialog->backward, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), dialog->case_sensitive, FALSE, FALSE, 0);
+
+	gtk_box_pack_start (GTK_BOX (dialog->dialog->vbox), dialog->entry, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (dialog->dialog->vbox), hbox, FALSE, FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog->dialog->vbox), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog->dialog), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 	gtk_widget_show (dialog->entry);
 	gtk_widget_show_all (hbox);
-	
+
 	gnome_window_icon_set_from_file (GTK_WINDOW (dialog->dialog), ICONDIR "/search-24.png");
 
-	gnome_dialog_button_connect (dialog->dialog, 0, search_cb, dialog);
-	gnome_dialog_close_hides (dialog->dialog, TRUE);
-	gnome_dialog_set_close (dialog->dialog, TRUE);
-
-	gnome_dialog_set_default (dialog->dialog, 0);
 	gtk_widget_grab_focus (dialog->entry);
 
-	gtk_signal_connect (GTK_OBJECT (dialog->entry), "changed",
-			    entry_changed, dialog);
-	gtk_signal_connect (GTK_OBJECT (dialog->entry), "activate",
-			    entry_activate, dialog);
+	g_signal_connect (dialog->dialog, "response", G_CALLBACK (search_dialog_response), dialog);
+	g_signal_connect (dialog->entry, "changed", G_CALLBACK (entry_changed), dialog);
+	g_signal_connect (dialog->entry, "activate", G_CALLBACK (entry_activate), dialog);
 
 	return dialog;
 }
@@ -105,19 +110,17 @@ gtk_html_search_dialog_new (GtkHTML *html)
 void
 gtk_html_search_dialog_destroy (GtkHTMLSearchDialog *d)
 {
+	gtk_widget_destroy (GTK_WIDGET (d->dialog));
 	g_free (d);
 }
 
 void
 search (GtkHTMLControlData *cd, gboolean regular)
 {
+	cd->regular = regular;
 	RUN_DIALOG (search, regular ? _("Find Regular Expression") :  _("Find"));
-
-	if (cd->search_dialog)
-		cd->search_dialog->regular = regular;
-
-	if (cd->search_dialog)
-		gtk_widget_grab_focus (cd->search_dialog->entry);
+	gtk_html_search_dialog_destroy (cd->search_dialog);
+	cd->search_dialog = NULL;
 }
 
 void
