@@ -24,6 +24,7 @@
 
 #include <sys/types.h>
 #include <gtk/gtklayout.h>
+#include <gtk/gtkbindings.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-job.h>
 
@@ -103,6 +104,8 @@ struct _GtkHTMLClass {
 
 	/* properties */
 	GtkHTMLClassProperties *properties;
+	GtkBindingSet *emacs_bindings;
+	gboolean use_emacs_bindings;
 };
 
 struct _GtkHTMLEditorAPI
@@ -111,7 +114,7 @@ struct _GtkHTMLEditorAPI
 	gboolean  (* check_word)              (GtkHTML *html, const gchar *word, gpointer data);
 	void      (* suggestion_request)      (GtkHTML *html, gpointer data);
 	void      (* add_to_session)          (GtkHTML *html, const gchar *word, gpointer data);
-	void      (* add_to_personal)         (GtkHTML *html, const gchar *word, gpointer data);
+	void      (* add_to_personal)         (GtkHTML *html, const gchar *word, const gchar *language, gpointer data);
 
 	/* unhandled commands */
 	gboolean  (* command)                 (GtkHTML *html, GtkHTMLCommandType com_type, gpointer data);
@@ -283,8 +286,6 @@ void                       gtk_html_drop_undo                     (GtkHTML
 /* DEPRECATED */
 #if 1
 gboolean                   gtk_html_build_with_gconf              (void);
-void                       gtk_html_set_default_background_color  (GtkHTML                   *html,
-								   GdkColor                  *c);
 void                       gtk_html_set_default_content_type      (GtkHTML                   *html,
 								   gchar                     *content_type);
 GtkWidget                 *gtk_html_new_from_string               (const gchar               *Astr,
