@@ -35,35 +35,13 @@ struct _HTMLImagePointer {
 	gint refcount;
 	gchar *url;
 	GdkPixbufLoader *loader;
-	GdkPixbuf *pixbuf;
 	GdkPixbufAnimation *animation;
+	GdkPixbufAnimationIter *iter;
 	GSList *interests; /* A list of HTMLImage's, or a NULL pointer for the background pixmap */
 	HTMLImageFactory *factory;
 	gint stall;
 	guint stall_timeout;
-};
-
-struct _HTMLImageAnimation {
-	/* stuff for animated image */
-	/* draw coordination */
-	gint x;
-	gint y;
-
-	/* remember engine offset */
-	gint ex, ey;
-
-	/* current frame */
-	GList *cur_frame;
-	gint cur_n;
-
-	/* animation timeout function */
-	gint timeout;
-
-	/* helper buffer */
-	GdkPixbuf *pixbuf;
-
-	/* active draw flag */
-	gint active;
+	guint animation_timeout;
 };
 
 #define HTML_IMAGE(x) ((HTMLImage *)(x))
@@ -72,7 +50,6 @@ struct _HTMLImageAnimation {
 struct _HTMLImage {
 	HTMLObject object;
         HTMLImagePointer *image_ptr;
-	HTMLImageAnimation *animation;
 
 	HTMLColor *color;
 
@@ -85,6 +62,7 @@ struct _HTMLImage {
 	gboolean percent_width  : 1;
 	gboolean percent_height : 1;
 	gboolean ismap : 1;
+	gboolean animation_active : 1;
 
 	gint8 hspace;
 	gint8 vspace;
@@ -160,21 +138,29 @@ gchar      *html_image_resolve_image_url  (GtkHTML          *html,
 
 void        html_image_edit_set_url        (HTMLImage       *image,
 					    const gchar     *url);
-
+guint       html_image_get_actual_width    (HTMLImage       *image,
+					    HTMLPainter     *painter);
+guint       html_image_get_actual_height   (HTMLImage       *image,
+					    HTMLPainter     *painter);
 /* FIXME move to htmlimagefactory.c */
-HTMLImageFactory *html_image_factory_new      (HTMLEngine       *e);
-void              html_image_factory_free     (HTMLImageFactory *factory);
-void              html_image_factory_cleanup  (HTMLImageFactory *factory); /* Does gc etc. - removes unused image entries */
-void              html_image_factory_stop_animations (HTMLImageFactory *factory);
-void              html_image_factory_deactivate_animations (HTMLImageFactory *factory);
-
-HTMLImagePointer *html_image_factory_register    (HTMLImageFactory *factory,
-						  HTMLImage        *i,
-						  const char       *filename,
-						  gboolean          reload);
-void              html_image_factory_unregister  (HTMLImageFactory *factory,
-						  HTMLImagePointer *pointer,
-						  HTMLImage        *i);
-void              html_image_factory_move_images (HTMLImageFactory *dst,
-						  HTMLImageFactory *src);
+HTMLImageFactory *html_image_factory_new                    (HTMLEngine       *e);
+void              html_image_factory_free                   (HTMLImageFactory *factory);
+void              html_image_factory_cleanup                (HTMLImageFactory *factory); /* Does gc etc. - removes unused image entries */
+void              html_image_factory_stop_animations        (HTMLImageFactory *factory);
+void              html_image_factory_deactivate_animations  (HTMLImageFactory *factory);
+HTMLImagePointer *html_image_factory_register               (HTMLImageFactory *factory,
+							     HTMLImage        *i,
+							     const char       *filename,
+							     gboolean          reload);
+void              html_image_factory_unregister             (HTMLImageFactory *factory,
+							     HTMLImagePointer *pointer,
+							     HTMLImage        *i);
+void              html_image_factory_move_images            (HTMLImageFactory *dst,
+							     HTMLImageFactory *src);
+void              html_image_factory_ref_all_images         (HTMLImageFactory *factory);
+void              html_image_factory_unref_all_images       (HTMLImageFactory *factory);
+void              html_image_factory_ref_image_ptr          (HTMLImageFactory *factory,
+							     const gchar      *url);
+void              html_image_factory_unref_image_ptr        (HTMLImageFactory *factory,
+							     const gchar      *url);
 #endif /* _HTMLIMAGE_H_ */
