@@ -70,6 +70,14 @@ struct _HTMLObject {
 	GData *object_data;
 };
 
+struct _HTMLObjectClearRectangle {
+	HTMLObject *object;
+	gint x;
+	gint y;
+	gint width;
+	gint height;
+};
+
 struct _HTMLObjectClass {
 	HTMLType type;
 
@@ -84,6 +92,7 @@ struct _HTMLObjectClass {
 
 	/* copy/cut/paste operations */
 	HTMLObject * (* op_copy)         (HTMLObject *self,
+					  HTMLObject *parent,
 					  HTMLEngine *e,
 					  GList      *from,
 					  GList      *to,
@@ -98,8 +107,9 @@ struct _HTMLObjectClass {
 	gboolean     (* merge)           (HTMLObject *self,
 					  HTMLObject *o,
 					  HTMLEngine *e,
-					  GList      *left,
-					  GList      *right);
+					  GList      **left,
+					  GList      **right,
+					  HTMLCursor *cursor);
 	void         (* remove_child)    (HTMLObject *self,
 					  HTMLObject *child);
 	void         (* split)           (HTMLObject *self,
@@ -113,14 +123,13 @@ struct _HTMLObjectClass {
 	/* Layout management and geometry handling.  */
 
 	HTMLFitType (* fit_line) (HTMLObject *o, HTMLPainter *painter,
-				  gboolean start_of_line, gboolean first_run,
+				  gboolean start_of_line, gboolean first_run, gboolean next_to_floating,
 				  gint width_left);
-	gboolean (* calc_size) (HTMLObject *o, HTMLPainter *painter);
+	gboolean (* calc_size) (HTMLObject *o, HTMLPainter *painter, GList **changed_objs);
 	gint (* calc_min_width) (HTMLObject *o, HTMLPainter *painter);
 	gint (* calc_preferred_width) (HTMLObject *o, HTMLPainter *painter);
-	void (* set_max_ascent) (HTMLObject *o, HTMLPainter *painter, gint a);
-	void (* set_max_descent) (HTMLObject *o, HTMLPainter *painter, gint d);
 	void (* set_max_width) (HTMLObject *o, HTMLPainter *painter, gint max_width);
+	void (* set_max_height) (HTMLObject *o, HTMLPainter *painter, gint max_height);
 
 	/* Relayout object `o' starting from child `child'.  This
            method can be called by the child when it changes any of
@@ -161,18 +170,19 @@ struct _HTMLObjectClass {
            HTMLTable does not derive from HTMLClue and we don't want
            to spend time reorganizing the hierarchy now.  */
 
-	gint (* get_left_margin) (HTMLObject *self, gint y);
-	gint (* get_right_margin) (HTMLObject *self, gint y);
+	gint (* get_left_margin) (HTMLObject *self, HTMLPainter *painter, gint y, gboolean with_aligned);
+	gint (* get_right_margin) (HTMLObject *self, HTMLPainter *painter, gint y, gboolean with_aligned);
 
-	void (* set_painter) (HTMLObject *o, HTMLPainter *painter, gint max_width);
+	void (* set_painter) (HTMLObject *o, HTMLPainter *painter);
 
 	/* Resetting the object.  Do this before using a different
            HTMLPainter.  */
 
 	void (* reset) (HTMLObject *o);
 
-	const gchar * (* get_url) (HTMLObject *o);
+	const gchar * (* get_url)    (HTMLObject *o);
 	const gchar * (* get_target) (HTMLObject *o);
+	const gchar * (* get_src)    (HTMLObject *o);
 
 	HTMLAnchor * (* find_anchor) (HTMLObject *o, const gchar *name, gint *x, gint *y);
 
@@ -190,7 +200,7 @@ struct _HTMLObjectClass {
            the returned value is `HTML_VALIGN_BOTTOM', the bottom of the object
            is aligned to the base line; if the value is `HTML_VALIGN_TOP', the
            top of the object is aligned to the top of the line; if the value is
-           `HTML_VALIGN_CENTER', the center of the object is aligned to the
+           `HTML_VALIGN_MIDDLE', the center of the object is aligned to the
            baseline.  */
 	HTMLVAlignType (* get_valign) (HTMLObject *self);
 
@@ -227,10 +237,10 @@ struct _HTMLObjectClass {
 
 	/* links */
 	HTMLObject * (* set_link)        (HTMLObject *self, HTMLColor *color, const gchar *url, const gchar *target);
-	HTMLObject * (* remove_link)     (HTMLObject *self, HTMLColor *color);
 
 	/* length */
 	guint        (* get_length)                (HTMLObject *self);
+	guint        (* get_line_length)           (HTMLObject *self, HTMLPainter *p, gint line_offset);
 	guint        (* get_recursive_length)      (HTMLObject *self);
 
 	/* movement */
@@ -238,6 +248,12 @@ struct _HTMLObjectClass {
 	HTMLObject * (* prev)            (HTMLObject *self, HTMLObject *child);
 	HTMLObject * (* head)            (HTMLObject *self);
 	HTMLObject * (* tail)            (HTMLObject *self);
+
+	HTMLClearType (* get_clear)      (HTMLObject *self);
+
+	gint (* get_n_children)          (HTMLObject *self);
+	HTMLObject * (* get_child)       (HTMLObject *self, gint index);
+	gint (* get_child_index)         (HTMLObject *self, HTMLObject *child);
 };
 
 
@@ -259,6 +275,7 @@ HTMLObject     *html_object_dup                   (HTMLObject            *self);
 
 /* copy/cut/paste operations */
 HTMLObject     *html_object_op_copy               (HTMLObject            *self,
+						   HTMLObject            *parent,
 						   HTMLEngine            *e,
 						   GList                 *from,
 						   GList                 *to,
@@ -273,8 +290,9 @@ HTMLObject     *html_object_op_cut                (HTMLObject            *self,
 gboolean        html_object_merge                 (HTMLObject            *self,
 						   HTMLObject            *with,
 						   HTMLEngine            *e,
-						   GList                 *left,
-						   GList                 *right);
+						   GList                 **left,
+						   GList                 **right,
+						   HTMLCursor            *cursor);
 void            html_object_remove_child          (HTMLObject            *self,
 						   HTMLObject            *child);
 void            html_object_split                 (HTMLObject            *self,
@@ -287,17 +305,23 @@ void            html_object_split                 (HTMLObject            *self,
 void            html_object_set_parent            (HTMLObject            *self,
 						   HTMLObject            *parent);
 gint            html_object_get_left_margin       (HTMLObject            *self,
-						   gint                   y);
+						   HTMLPainter           *painter,
+						   gint                   y,
+						   gboolean               with_aligned);
 gint            html_object_get_right_margin      (HTMLObject            *self,
-						   gint                   y);
+						   HTMLPainter           *painter,
+						   gint                   y,
+						   gboolean               with_aligned);
 void            html_object_set_painter           (HTMLObject            *o,
-						   HTMLPainter           *p,
-						   gint                   max_width);
+						   HTMLPainter           *p);
 void            html_object_clear_word_width      (HTMLObject            *o);
 void            html_object_reset                 (HTMLObject            *o);
 gboolean        html_object_is_text               (HTMLObject            *object);
+gboolean        html_object_is_clue               (HTMLObject            *object);
 HTMLEngine     *html_object_get_engine            (HTMLObject            *self,
 						   HTMLEngine            *e);
+HTMLEngine     *html_object_engine                (HTMLObject            *o,
+						   HTMLEngine            *e);
 void            html_object_forall                (HTMLObject            *self,
 						   HTMLEngine            *e,
 						   HTMLObjectForallFunc   func,
@@ -315,6 +339,7 @@ HTMLObject     *html_object_prev_by_type          (HTMLObject            *self,
 						   HTMLType               t);
 HTMLObject     *html_object_nth_parent            (HTMLObject            *self,
 						   gint                   n);
+gint            html_object_get_parent_level      (HTMLObject            *self);
 /* do search request on object using info */
 gboolean        html_object_search                (HTMLObject            *self,
 						   HTMLSearch            *info);
@@ -349,18 +374,17 @@ HTMLFitType     html_object_fit_line              (HTMLObject            *o,
 						   HTMLPainter           *painter,
 						   gboolean               start_of_line,
 						   gboolean               first_run,
+						   gboolean               next_to_floating,
 						   gint                   width_left);
 gboolean        html_object_calc_size             (HTMLObject            *o,
-						   HTMLPainter           *painter);
-void            html_object_set_max_ascent        (HTMLObject            *o,
 						   HTMLPainter           *painter,
-						   gint                   a);
-void            html_object_set_max_descent       (HTMLObject            *o,
-						   HTMLPainter           *painter,
-						   gint                   d);
+						   GList                **changed_objs);
 void            html_object_set_max_width         (HTMLObject            *o,
 						   HTMLPainter           *painter,
 						   gint                   max_width);
+void            html_object_set_max_height        (HTMLObject            *o,
+						   HTMLPainter           *painter,
+						   gint                   max_height);
 gint            html_object_calc_min_width        (HTMLObject            *o,
 						   HTMLPainter           *painter);
 gint            html_object_calc_preferred_width  (HTMLObject            *o,
@@ -368,9 +392,9 @@ gint            html_object_calc_preferred_width  (HTMLObject            *o,
 void            html_object_calc_abs_position     (HTMLObject            *o,
 						   gint                  *x_return,
 						   gint                  *y_return);
-void            html_object_calc_intersection     (HTMLObject            *o,
-						   ArtIRect              *intersection,
-						   gint                   x,
+gboolean        html_object_intersect             (HTMLObject            *o,
+					           GdkRectangle          *intersection,
+					           gint                   x,
 						   gint                   y,
 						   gint                   width,
 						   gint                   height);
@@ -382,6 +406,8 @@ HTMLVAlignType  html_object_get_valign            (HTMLObject            *self);
 /* Links.  */
 const gchar    *html_object_get_url               (HTMLObject            *o);
 const gchar    *html_object_get_target            (HTMLObject            *o);
+gchar          *html_object_get_complete_url      (HTMLObject            *o);
+const gchar    *html_object_get_src               (HTMLObject            *o);
 HTMLAnchor     *html_object_find_anchor           (HTMLObject            *o,
 						   const gchar           *name,
 						   gint                  *x,
@@ -408,6 +434,9 @@ void            html_object_get_cursor_base       (HTMLObject            *obj,
 						   gint                  *x,
 						   gint                  *y);
 guint           html_object_get_length            (HTMLObject            *self);
+guint           html_object_get_line_length       (HTMLObject            *self,
+						   HTMLPainter           *p,
+						   gint                   line_offset);
 guint           html_object_get_recursive_length  (HTMLObject            *self);
 guint           html_object_get_bytes             (HTMLObject            *self);
 guint           html_object_get_index             (HTMLObject            *self,
@@ -462,7 +491,8 @@ gboolean    html_object_select_range             (HTMLObject *self,
 						  gboolean    queue_draw);
 void        html_object_append_selection_string  (HTMLObject *self,
 						  GString    *buffer);
-gchar      *html_object_get_selection_string     (HTMLObject *o);
+gchar      *html_object_get_selection_string     (HTMLObject *o,
+						  HTMLEngine *e);
 
 /* Saving.  */
 gboolean  html_object_save  (HTMLObject          *self,
@@ -482,13 +512,19 @@ void  html_object_change_set_down  (HTMLObject      *self,
 
 /* object data */
 
-void      html_object_set_data               (HTMLObject  *object,
-					      const gchar *key,
-					      gpointer     data);
-gpointer  html_object_get_data               (HTMLObject  *object,
-					      const gchar *key);
-void      html_object_copy_data_from_object  (HTMLObject  *dst,
-					      HTMLObject  *src);
+void      html_object_set_data               (HTMLObject          *object,
+					      const gchar         *key,
+					      const gchar         *value);
+void      html_object_set_data_full          (HTMLObject          *object,
+					      const gchar         *key,
+					      const gpointer       value,
+					      GDestroyNotify       func);
+gpointer  html_object_get_data               (HTMLObject          *object,
+					      const gchar         *key);
+void      html_object_copy_data_from_object  (HTMLObject          *dst,
+					      HTMLObject          *src);
+gboolean  html_object_save_data              (HTMLObject          *self,
+					      HTMLEngineSaveState *state);
 /*
  * editing
 */
@@ -505,5 +541,45 @@ void      html_object_check_cut_lists            (HTMLObject *self,
 						  HTMLObject *replacement,
 						  GList      *left,
 						  GList      *right);
+GList    *html_object_heads_list                 (HTMLObject *o);
+GList    *html_object_tails_list                 (HTMLObject *o);
+void      html_object_merge_down                 (HTMLObject *o,
+						  HTMLObject *w,
+						  HTMLEngine *e);
+gboolean  html_object_is_parent                  (HTMLObject *parent,
+						  HTMLObject *child);
+gint      html_object_get_insert_level           (HTMLObject *o);
 
+void      html_object_engine_translation   (HTMLObject *o,
+					    HTMLEngine *e,
+					    gint       *tx,
+					    gint       *ty);
+gboolean  html_object_engine_intersection  (HTMLObject *o,
+					    HTMLEngine *e,
+					    gint        tx,
+					    gint        ty,
+					    gint       *x1,
+					    gint       *y1,
+					    gint       *x2,
+					    gint       *y2);
+
+void  html_object_add_to_changed  (GList      **changed_objs,
+				   HTMLObject  *o);
+
+gint html_object_get_n_children (HTMLObject *self);
+HTMLObject * html_object_get_child (HTMLObject *self, gint index);
+gint html_object_get_child_index (HTMLObject *self, HTMLObject *child);
+
+HTMLClearType html_object_get_clear (HTMLObject *self);
+
+HTMLObject *html_object_next_cursor_object  (HTMLObject *o,
+					     HTMLEngine *e,
+					     gint       *offset);
+HTMLObject *html_object_prev_cursor_object  (HTMLObject *o,
+					     HTMLEngine *e,
+					     gint       *offset);
+HTMLObject *html_object_next_cursor_leaf    (HTMLObject *o,
+					     HTMLEngine *e);
+HTMLObject *html_object_prev_cursor_leaf    (HTMLObject *o,
+					     HTMLEngine *e);
 #endif /* _HTMLOBJECT_H_ */
