@@ -22,16 +22,18 @@
 
 #include <gnome.h>
 #include <gtk/gtk.h>
+#include <gtk/gtkbutton.h>
 
 #include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-master.h>
-#include <libgnomeprint/gnome-print-master-preview.h>
+#include <libgnomeprint/gnome-print-job.h>
+#include <libgnomeprintui/gnome-print-job-preview.h>
 
 #include <libsoup/soup.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
+#include <string.h>
 
 #include "config.h"
 #include "gtkhtml.h"
@@ -80,7 +82,8 @@ static gchar *parse_href (const gchar *s);
 
 static GtkHTML *html;
 static GtkHTMLStream *html_stream_handle = NULL;
-static GtkWidget *animator, *entry;
+/* static GtkWidget *animator; */
+static GtkWidget *entry;
 static GtkWidget *popup_menu, *popup_menu_back, *popup_menu_forward, *popup_menu_home;
 static GtkWidget *toolbar_back, *toolbar_forward;
 static HTMLURL *baseURL = NULL;
@@ -90,7 +93,8 @@ static int go_position;
 
 static gboolean use_redirect_filter;
 
-static gboolean slow_loading = FALSE, exit_when_done = FALSE;
+static gboolean slow_loading = FALSE;
+static gboolean exit_when_done = FALSE;
 
 static gint redirect_timerId = 0;
 static gchar *redirect_url = NULL;
@@ -169,64 +173,61 @@ static void
 create_toolbars (GtkWidget *app)
 {
 	GtkWidget *dock;
-	GtkWidget *hbox, *hbox2;
+	GtkWidget *hbox;
 	GtkWidget *frame;
-	GtkWidget *button;
 	GtkWidget *toolbar;
-	char *imgloc;
+	/* char *imgloc; */
 
-	dock = gnome_dock_item_new ("testgtkhtml-toolbar1",
-				    (GNOME_DOCK_ITEM_BEH_EXCLUSIVE));
+	dock = bonobo_dock_item_new ("testgtkhtml-toolbar1",
+				    (BONOBO_DOCK_ITEM_BEH_EXCLUSIVE));
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (dock), hbox);
-	gtk_container_border_width (GTK_CONTAINER (dock), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (dock), 2);
 	
-	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,
-				   GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar),
-			       GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_SPACE_LINE);
+	toolbar = gtk_toolbar_new ();
+	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
+	/* RM2 gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_SPACE_LINE);
 	gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar),
-				       GTK_RELIEF_NONE);
+	GTK_RELIEF_NONE); */
 	gtk_box_pack_start (GTK_BOX (hbox), toolbar, FALSE, FALSE, 0);
 
 	toolbar_back = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-				 NULL,
-				 "Move back",
-				 "Back",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_BACK),
-				 back_cb, NULL);
+						NULL,
+						"Move back",
+						"Back",
+						gtk_image_new_from_stock ("gtk-go-back", GTK_ICON_SIZE_LARGE_TOOLBAR),
+						GTK_SIGNAL_FUNC (back_cb), NULL);
 	gtk_widget_set_sensitive(toolbar_back, FALSE);
 	toolbar_forward = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-				 NULL,
-				 "Move forward",
-				 "Forward",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_FORWARD),
-				 forward_cb, NULL);
+						   NULL,
+						   "Move forward",
+						   "Forward",
+						   gtk_image_new_from_stock ("gtk-go-forward", GTK_ICON_SIZE_LARGE_TOOLBAR),
+						   GTK_SIGNAL_FUNC (forward_cb), NULL);
 	gtk_widget_set_sensitive(toolbar_forward, FALSE);
 	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
 				 NULL,
 				 "Stop loading",
 				 "Stop",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_STOP),
-				 stop_cb,
+				 gtk_image_new_from_stock ("gtk-stop", GTK_ICON_SIZE_LARGE_TOOLBAR),
+				 GTK_SIGNAL_FUNC (stop_cb),
 				 NULL);
 	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
 				 NULL,
 				 "Reload page",
 				 "Reload",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_REFRESH),
-				 reload_cb, NULL);
+				 gtk_image_new_from_stock ("gtk-refresh", GTK_ICON_SIZE_LARGE_TOOLBAR),
+				 GTK_SIGNAL_FUNC (reload_cb), NULL);
 	gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
 				 NULL,
 				 "Home page",
 				 "Home",
-				 gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_HOME),
-				 home_cb, NULL);
+				 gtk_image_new_from_stock ("gtk-home", GTK_ICON_SIZE_LARGE_TOOLBAR),
+				 GTK_SIGNAL_FUNC (home_cb), NULL);
 	gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
 
-	animator = gnome_animator_new_with_size (32, 32);
+	/* animator = gnome_animator_new_with_size (32, 32);
 
 	if (g_file_exists("32.png"))
 	  imgloc = "32.png";
@@ -239,40 +240,32 @@ create_toolbars (GtkWidget *app)
 							0, 0,
 							25,
 							32, 
-							32, 32);
+							32, 32); */
 
 	frame = gtk_frame_new (NULL);
-	gtk_container_add (GTK_CONTAINER (frame), animator);
+	/* TODO2 gtk_container_add (GTK_CONTAINER (frame), animator); */
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
 	gtk_box_pack_end (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
-	gnome_animator_set_loop_type (GNOME_ANIMATOR (animator), 
-				      GNOME_ANIMATOR_LOOP_RESTART);
+	/* gnome_animator_set_loop_type (GNOME_ANIMATOR (animator), 
+	   GNOME_ANIMATOR_LOOP_RESTART); */
 	gtk_widget_show_all (dock);
-	gnome_dock_add_item (GNOME_DOCK (GNOME_APP (app)->dock),
-			     GNOME_DOCK_ITEM (dock), GNOME_DOCK_TOP, 1, 0, 0, FALSE);
+	bonobo_dock_add_item (BONOBO_DOCK (GNOME_APP (app)->dock),
+			      BONOBO_DOCK_ITEM (dock), BONOBO_DOCK_TOP, 1, 0, 0, FALSE);
 
 	/* Create the location bar */
-	dock = gnome_dock_item_new ("testgtkhtml-toolbar2",
-				    (GNOME_DOCK_ITEM_BEH_EXCLUSIVE));
+	dock = bonobo_dock_item_new ("testgtkhtml-toolbar2",
+				     (BONOBO_DOCK_ITEM_BEH_EXCLUSIVE));
 	hbox = gtk_hbox_new (FALSE, 2);
 	gtk_container_add (GTK_CONTAINER (dock), hbox);
-	gtk_container_border_width (GTK_CONTAINER (dock), 2);
-	hbox2 = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox2), gtk_label_new ("Goto"), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox2), gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_REDO), FALSE, FALSE, 0);
-	button = gtk_button_new ();
-	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	gtk_container_add (GTK_CONTAINER (button), hbox2);
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (dock), 2);
 	gtk_box_pack_start (GTK_BOX (hbox),
 			    gtk_label_new ("Location:"), FALSE, FALSE, 0);
 	entry = gtk_entry_new ();
-	gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(entry_goto_url), NULL);
+	g_signal_connect (entry, "activate", G_CALLBACK (entry_goto_url), NULL);
 	gtk_box_pack_start (GTK_BOX (hbox),
 			    entry, TRUE, TRUE, 0);
-	gnome_dock_add_item (GNOME_DOCK (GNOME_APP (app)->dock),
-			     GNOME_DOCK_ITEM (dock), GNOME_DOCK_TOP, 2, 0, 0, FALSE);
+	bonobo_dock_add_item (BONOBO_DOCK (GNOME_APP (app)->dock),
+			      BONOBO_DOCK_ITEM (dock), BONOBO_DOCK_TOP, 2, 0, 0, FALSE);
 
 }
 
@@ -284,7 +277,7 @@ print_footer (GtkHTML *html, GnomePrintContext *context,
 	      gdouble x, gdouble y, gdouble width, gdouble height, gpointer user_data)
 {
 	gchar *text = g_strdup_printf ("- %d of %d -", page_num, pages);
-	gdouble tw = gnome_font_get_width_string (font, "text");
+	gdouble tw = gnome_font_get_width_utf8 (font, "text");
 
 	if (font) {
 		gnome_print_newpath     (context);
@@ -302,29 +295,31 @@ static void
 print_preview_cb (GtkWidget *widget,
 		  gpointer data)
 {
-	GnomePrintMaster *print_master;
+	GnomePrintJob *print_master;
 	GnomePrintContext *print_context;
 	GtkWidget *preview;
 
-	print_master = gnome_print_master_new ();
-	gnome_print_master_set_paper (print_master, gnome_paper_with_name ("US-Letter"));
+	print_master = gnome_print_job_new (NULL);
+	/* FIX2 gnome_print_master_set_paper (print_master, gnome_paper_with_name ("US-Letter")); */
 	gtk_html_print_set_master (html, print_master);
 
-	print_context = gnome_print_master_get_context (print_master);
+	print_context = gnome_print_job_get_context (print_master);
+	font = gnome_font_find_closest ("Helvetica", 12);
 
 	page_num = 1;
 	pages = gtk_html_print_get_pages_num (html, print_context,
 					      .0, gnome_font_get_ascender (font) + gnome_font_get_descender (font));
-	font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, FALSE, 12);
 	gtk_html_print_with_header_footer (html, print_context,
 					   .0, gnome_font_get_ascender (font) + gnome_font_get_descender (font),
 					   NULL, print_footer, NULL);
-	if (font) gtk_object_unref (GTK_OBJECT (font));
-
-	preview = GTK_WIDGET (gnome_print_master_preview_new (print_master, "HTML Print Preview"));
+	if (font)
+		g_object_unref (font);
+	
+	gnome_print_job_close (print_master);
+	preview = gnome_print_job_preview_new (print_master, "Print Preview");
 	gtk_widget_show (preview);
 
-	gtk_object_unref (GTK_OBJECT (print_master));
+	g_object_unref (print_master);
 }
 
 static void
@@ -363,7 +358,7 @@ static void
 redraw_cb (GtkWidget *widget, gpointer data)
 {
 	g_print ("forcing redraw\n");
-	gtk_widget_draw (GTK_WIDGET (html), NULL);
+	gtk_widget_queue_draw (GTK_WIDGET (html));
 }
 
 static void
@@ -481,11 +476,11 @@ stop_cb (GtkWidget *widget, gpointer data)
 static void
 load_done (GtkHTML *html)
 {
-	gnome_animator_stop (GNOME_ANIMATOR (animator));
+	/* TODO2 gnome_animator_stop (GNOME_ANIMATOR (animator));
 	gnome_animator_goto_frame (GNOME_ANIMATOR (animator), 1);
 
 	if (exit_when_done)
-		gtk_main_quit();
+	gtk_main_quit(); */
 }
 
 static int
@@ -805,7 +800,7 @@ goto_url(const char *url, int back_or_forward)
 		redirect_timerId = 0;
 	}
 
-	gnome_animator_start (GNOME_ANIMATOR (animator));
+	/* TODO2 gnome_animator_start (GNOME_ANIMATOR (animator)); */
 
 	use_redirect_filter = TRUE;
 
@@ -873,15 +868,15 @@ goto_url(const char *url, int back_or_forward)
 			item = g_list_nth_data(go_list, i);
 			item->widget = gtk_radio_menu_item_new_with_label(group, item->url);
 
-			gtk_signal_connect (GTK_OBJECT (item->widget), "activate",
-					    GTK_SIGNAL_FUNC (go_list_cb), GINT_TO_POINTER (i));
+			g_signal_connect (item->widget, "activate",
+					  G_CALLBACK (go_list_cb), GINT_TO_POINTER (i));
 
-			group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item->widget));
+			group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(item->widget));
 
 			if(i == 0)
 				gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item->widget), TRUE);
 
-			gtk_menu_append(GTK_MENU(GTK_MENU_ITEM(main_menu[3].widget)->submenu), item->widget);
+			gtk_menu_shell_append (GTK_MENU_SHELL (GTK_MENU_ITEM(main_menu[3].widget)->submenu), item->widget);
 			gtk_widget_show(item->widget);
 			
 		}
@@ -948,33 +943,29 @@ main (gint argc, gchar *argv[])
 	GtkWidget *html_widget;
 	GtkWidget *scrolled_window;
 	/* GdkColor   bgColor = {0, 0xc7ff, 0xc7ff, 0xc7ff}; */
-	poptContext ctx;
-#ifdef GTKHTML_HAVE_GCONF
+	/* RM2 poptContext ctx; */
 	GError  *gconf_error  = NULL;
-#endif
 
 #ifdef MEMDEBUG
 	void *p = malloc (1024);	/* to make linker happy with ccmalloc */
 #endif
-	gnome_init_with_popt_table (PACKAGE, VERSION,
-				    argc, argv, options, 0, &ctx);
-#ifdef GTKHTML_HAVE_GCONF
+	/* gnome_init_with_popt_table (PACKAGE, VERSION,
+	   argc, argv, options, 0, &ctx); */
+	gnome_program_init ("testgtkhtml", VERSION, LIBGNOMEUI_MODULE, argc, argv,
+			    GNOME_PARAM_POPT_TABLE, options,
+			    NULL);
 	if (!gconf_init (argc, argv, &gconf_error)) {
 		g_assert (gconf_error != NULL);
 		g_error ("GConf init failed:\n  %s", gconf_error->message);
 		return FALSE;
 	}
-#endif
 
-	gdk_rgb_init ();
-	
-	gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
-	gtk_widget_set_default_visual (gdk_rgb_get_visual ());
-	
+	/* RM2 gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
+	   gtk_widget_set_default_visual (gdk_rgb_get_visual ()); */
+
 	app = gnome_app_new ("testgtkhtml", "GtkHTML: testbed application");
 
-	gtk_signal_connect (GTK_OBJECT (app), "delete_event",
-			    GTK_SIGNAL_FUNC (exit_cb), NULL);
+	g_signal_connect (app, "delete_event", G_CALLBACK (exit_cb), NULL);
 
 	create_toolbars (app);
 	bar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
@@ -1007,50 +998,37 @@ main (gint argc, gchar *argv[])
 	/* Create a popup menu with disabled back and forward items */
 	popup_menu = gtk_menu_new();
 
-	popup_menu_back = gtk_menu_item_new_with_label("Back");
-	gtk_widget_set_sensitive(popup_menu_back, FALSE);
-	gtk_menu_append(GTK_MENU(popup_menu), popup_menu_back);
-	gtk_widget_show(popup_menu_back);
-	gtk_signal_connect (GTK_OBJECT (popup_menu_back), "activate",
-			    GTK_SIGNAL_FUNC (back_cb), NULL);
+	popup_menu_back = gtk_menu_item_new_with_label ("Back");
+	gtk_widget_set_sensitive (popup_menu_back, FALSE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), popup_menu_back);
+	gtk_widget_show (popup_menu_back);
+	g_signal_connect (popup_menu_back, "activate", G_CALLBACK (back_cb), NULL);
 
-	popup_menu_forward = gtk_menu_item_new_with_label("Forward");
-	gtk_widget_set_sensitive(popup_menu_forward, FALSE);
-	gtk_menu_append(GTK_MENU(popup_menu), popup_menu_forward);
-	gtk_widget_show(popup_menu_forward);
-	gtk_signal_connect (GTK_OBJECT (popup_menu_forward), "activate",
-			    GTK_SIGNAL_FUNC (forward_cb), NULL);
+	popup_menu_forward = gtk_menu_item_new_with_label ("Forward");
+	gtk_widget_set_sensitive (popup_menu_forward, FALSE);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), popup_menu_forward);
+	gtk_widget_show (popup_menu_forward);
+	g_signal_connect (popup_menu_forward, "activate", G_CALLBACK (forward_cb), NULL);
 
-	popup_menu_home = gtk_menu_item_new_with_label("Home");
-	gtk_menu_append(GTK_MENU(popup_menu), popup_menu_home);
-	gtk_widget_show(popup_menu_home);
-	gtk_signal_connect (GTK_OBJECT (popup_menu_home), "activate",
-			    GTK_SIGNAL_FUNC (home_cb), NULL);
+	popup_menu_home = gtk_menu_item_new_with_label ("Home");
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), popup_menu_home);
+	gtk_widget_show (popup_menu_home);
+	g_signal_connect (popup_menu_home, "activate", G_CALLBACK (home_cb), NULL);
 
 	/* End of menu creation */
 
 	gtk_widget_set_events (html_widget, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 
-	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
-			    GTK_SIGNAL_FUNC (title_changed_cb), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "url_requested",
-			    GTK_SIGNAL_FUNC (url_requested), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "load_done",
-			    GTK_SIGNAL_FUNC (load_done), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "on_url",
-			    GTK_SIGNAL_FUNC (on_url), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "set_base",
-			    GTK_SIGNAL_FUNC (on_set_base), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
-			    GTK_SIGNAL_FUNC (on_button_press_event), popup_menu);
-	gtk_signal_connect (GTK_OBJECT (html), "link_clicked",
-			    GTK_SIGNAL_FUNC (on_link_clicked), NULL);
-	gtk_signal_connect (GTK_OBJECT (html), "redirect",
-			    GTK_SIGNAL_FUNC (on_redirect), NULL);
-	gtk_signal_connect (GTK_OBJECT (html), "submit",
-			    GTK_SIGNAL_FUNC (on_submit), NULL);
-	gtk_signal_connect (GTK_OBJECT (html), "object_requested",
-			    GTK_SIGNAL_FUNC (object_requested_cmd), NULL);
+	g_signal_connect (html, "title_changed", G_CALLBACK (title_changed_cb), (gpointer)app);
+	g_signal_connect (html, "url_requested", G_CALLBACK (url_requested), (gpointer)app);
+	g_signal_connect (html, "load_done", G_CALLBACK (load_done), (gpointer)app);
+	g_signal_connect (html, "on_url", G_CALLBACK (on_url), (gpointer)app);
+	g_signal_connect (html, "set_base", G_CALLBACK (on_set_base), (gpointer)app);
+	g_signal_connect (html, "button_press_event", G_CALLBACK (on_button_press_event), popup_menu);
+	g_signal_connect (html, "link_clicked", G_CALLBACK (on_link_clicked), NULL);
+	g_signal_connect (html, "redirect", G_CALLBACK (on_redirect), NULL);
+	g_signal_connect (html, "submit", G_CALLBACK (on_submit), NULL);
+	g_signal_connect (html, "object_requested", G_CALLBACK (object_requested_cmd), NULL);
 
 #if 0
 	gtk_box_pack_start_defaults (GTK_BOX (hbox), GTK_WIDGET (html));
@@ -1065,13 +1043,15 @@ main (gint argc, gchar *argv[])
 
 	gtk_widget_show_all (app);
 
-	{
+	if (argc > 1 && *argv [argc - 1] != '-')
+		goto_url (argv [argc - 1], 0);
+	/* RM2 {
 		const char **args;
 		args = poptGetArgs(ctx);
 
 		if (args && args[0])
 			goto_url (args[0], 0);
-	}
+			} */
 
 	gtk_main ();
 
